@@ -34,15 +34,35 @@
 #include "datetime.h"
 #include "datetime-dialog.h"
 
+/*
+ * Get date/time string
+ */
+gchar * datetime_do_utf8strftime(const char *format, const struct tm *tm)
+{
+  int len;
+  gchar buf[256];
+  gchar *utf8str = NULL;
+
+  /* get formatted date/time */
+  len = strftime(buf, sizeof(buf)-1, format, tm);
+  if (len == 0)
+    return g_strdup(_("Invalid format"));
+
+  buf[len] = '\0';  /* make sure nul terminated string */
+  utf8str = g_locale_to_utf8(buf, -1, NULL, NULL, NULL);
+  if(utf8str == NULL)
+    return g_strdup(_("Error"));
+
+  return utf8str;
+}
+
 /*
  * set date and time labels
  */
 gboolean datetime_update(gpointer data)
 {
   GTimeVal timeval;
-  gchar buf[256];
   gchar *utf8str;
-  int len;
   struct tm *current;
   t_datetime *datetime;
 
@@ -55,46 +75,46 @@ gboolean datetime_update(gpointer data)
 
   g_get_current_time(&timeval);
   current = localtime((time_t *)&timeval.tv_sec);
-  if (datetime->date_format != NULL && 
-      GTK_IS_LABEL(datetime->date_label) &&
-      strlen(datetime->date_format) > 0)
+  if (datetime->date_format != NULL && GTK_IS_LABEL(datetime->date_label))
   {
-    len = strftime(buf, sizeof(buf) - 1, datetime->date_format, current);
-    if (len != 0)
-    {
-      buf[len] = '\0';  /* make sure nul terminated string */
-      utf8str = g_locale_to_utf8(buf, len, NULL, NULL, NULL);
-      if (utf8str != NULL)
-      {
-	gtk_label_set_text(GTK_LABEL(datetime->date_label), utf8str);
-	g_free(utf8str);
-      }
-    }
-    else
-    {
-      gtk_label_set_text(GTK_LABEL(datetime->date_label), _("Error"));
-    }
+    utf8str = datetime_do_utf8strftime(datetime->date_format, current);
+    gtk_label_set_text(GTK_LABEL(datetime->date_label), utf8str);
+    g_free(utf8str);
   }
 
-  if (datetime->time_format != NULL && 
-      GTK_IS_LABEL(datetime->time_label) &&
-      strlen(datetime->time_format) > 0)
+  if (datetime->time_format != NULL && GTK_IS_LABEL(datetime->time_label))
   {
-    len = strftime(buf, sizeof(buf) - 1, datetime->time_format, current);
-    if (len != 0)
-    {
-      buf[len] = '\0';  /* make sure nul terminated string */
-      utf8str = g_locale_to_utf8(buf, len, NULL, NULL, NULL);
-      if (utf8str != NULL)
-      {
-	gtk_label_set_text(GTK_LABEL(datetime->time_label), utf8str);
-	g_free(utf8str);
-      }
-    }
-    else
-    {
-      gtk_label_set_text(GTK_LABEL(datetime->time_label), _("Error"));
-    }
+    utf8str = datetime_do_utf8strftime(datetime->time_format, current);
+    gtk_label_set_text(GTK_LABEL(datetime->time_label), utf8str);
+    g_free(utf8str);
+  }
+
+  /* hide labels based on layout-selection */
+  gtk_widget_show(GTK_WIDGET(datetime->time_label));
+  gtk_widget_show(GTK_WIDGET(datetime->date_label));
+  switch(datetime->layout)
+  {
+    case LAYOUT_DATE:
+      gtk_widget_hide(GTK_WIDGET(datetime->time_label));
+      break;
+    case LAYOUT_TIME:
+      gtk_widget_hide(GTK_WIDGET(datetime->date_label));
+      break;
+    default:
+      break;
+  }
+
+  /* set order based on layout-selection */
+  switch(datetime->layout)
+  {
+    case LAYOUT_DATE_TIME:
+      gtk_box_reorder_child(GTK_BOX(datetime->vbox), datetime->time_label, 1);
+      gtk_box_reorder_child(GTK_BOX(datetime->vbox), datetime->date_label, 0);
+      break;
+
+    default:
+      gtk_box_reorder_child(GTK_BOX(datetime->vbox), datetime->time_label, 0);
+      gtk_box_reorder_child(GTK_BOX(datetime->vbox), datetime->date_label, 1);
   }
   return TRUE;
 }
@@ -333,6 +353,17 @@ static void datetime_update_time_font(t_datetime *datetime)
   }
 }
 
+/*
+ * set layout after doing some checks
+ */
+void datetime_apply_layout(t_datetime *datetime, t_layout layout)
+{
+  if(0 <= layout && layout < LAYOUT_COUNT)
+  {
+    datetime->layout = layout;
+  }
+}
+
 /*
  * set the date and time font type
  */
@@ -415,8 +446,6 @@ static int datetime_set_size(XfcePanelPlugin *plugin,
     gint size,
     t_datetime *datetime)
 {
-  DBG("set_size called! with new size of %d", size);
-
   if(size > 26)
     gtk_container_set_border_width(GTK_CONTAINER(datetime->frame), 2);
   else
@@ -433,14 +462,16 @@ static void datetime_read_rc_file(XfcePanelPlugin *plugin, t_datetime *dt)
 {
   gchar *file;
   XfceRc *rc;
+  t_layout layout;
   const gchar *date_font, *time_font, *date_format, *time_format;
   const gchar *val;
 
   /* load defaults */
-  date_font = "Bitstream Vera Sans 8";
-  time_font = "Bitstream Vera Sans 10";
-  date_format = _("%Y-%m-%d");
-  time_format = _("%H:%M");
+  layout = LAYOUT_DATE_TIME;
+  date_font = g_strdup("Bitstream Vera Sans 8");
+  time_font = g_strdup("Bitstream Vera Sans 10");
+  date_format = g_strdup("%Y/%m/%d");
+  time_format = g_strdup("%H:%M");
 
   /* open file */
   if((file = xfce_panel_plugin_lookup_rc_file(plugin)) != NULL)
@@ -450,10 +481,11 @@ static void datetime_read_rc_file(XfcePanelPlugin *plugin, t_datetime *dt)
 
     if(rc != NULL)
     {
+      layout	  = xfce_rc_read_int_entry(rc, "layout", layout);
       val = xfce_rc_read_entry(rc, "date_font", date_font);
-      date_font          = g_strdup(val);
+      date_font	  = g_strdup(val);
       val = xfce_rc_read_entry(rc, "time_font", time_font);
-      time_font          = g_strdup(val);
+      time_font	  = g_strdup(val);
       val = xfce_rc_read_entry(rc, "date_format", date_format);
       date_format = g_strdup(val);
       val = xfce_rc_read_entry(rc, "time_format", time_format);
@@ -464,6 +496,7 @@ static void datetime_read_rc_file(XfcePanelPlugin *plugin, t_datetime *dt)
   }
 
   /* set values in dt struct */
+  datetime_apply_layout(dt, layout);
   datetime_apply_font(dt, date_font, time_font);
   datetime_apply_format(dt, date_format, time_format);
 }
@@ -482,6 +515,7 @@ void datetime_write_rc_file(XfcePanelPlugin *plugin, t_datetime *dt)
   rc = xfce_rc_simple_open(file, FALSE);
   g_free(file);
 
+  xfce_rc_write_int_entry(rc, "layout", dt->layout);
   xfce_rc_write_entry(rc, "date_font", dt->date_font);
   xfce_rc_write_entry(rc, "time_font", dt->time_font);
   xfce_rc_write_entry(rc, "date_format", dt->date_format);
@@ -547,9 +581,6 @@ static t_datetime * datetime_new(XfcePanelPlugin *plugin)
 
   /* call widget-create function */
   datetime_create_widget(datetime);
-  
-  /* display plugin */
-  gtk_widget_show_all(datetime->eventbox);
 
   /* set calendar variables */
   datetime->cal = NULL;
@@ -560,6 +591,9 @@ static t_datetime * datetime_new(XfcePanelPlugin *plugin)
   /* set date and time labels */
   datetime_update(datetime);
 
+  /* display plugin */
+  gtk_widget_show_all(datetime->eventbox);
+
   return datetime;
 }
 
